@@ -1,4 +1,4 @@
-mergeInto(LibraryManager.library, {
+addToLibrary({
   proxied_js_function__proxy: 'sync',
   proxied_js_function__sig: 'v',
   proxied_js_function: function() {
